#!/usr/bin/env python3
#
#

import influxdb_client
from influxdb_client.client.write_api import SYNCHRONOUS


DEST_MEASUREMENT="pf_analytics"
DEST_BUCKET="websites/analytics"
DEST_ORG=""


def query_and_write_agg_info(q, key, query_api, write_api, field_pref=''):
    ''' Run one of the map queries, pull out results and write the resulting aggregates back in
    
    '''
    print(q)
    tables = query_api.query(q)
    for table in tables:
        print(table)
        for row in table.records:
            print (row.values)
            p = influxdb_client.Point(DEST_MEASUREMENT)
            p.tag(key, row.values[key])
            p.tag("domain", row.values['domain'])
            p.tag("host", row.values['host'])
            if "action" in row.values:
                p.tag("action", row.values['action'])
            p.tag("page", row.values['page'])
            p.field(f"{field_pref}PageViews", row.values['_value'])
            p.time(row.values['_time'])
            write_api.write(DEST_BUCKET, DEST_ORG, p)


def query_and_write_field_val(q, key, query_api, write_api):
    ''' Run one of the map queries, pull out results and write the resulting aggregates back in
    
    '''
    print(q)
    tables = query_api.query(q)
    for table in tables:
        print(table)
        for row in table.records:
            print (row.values)
            p = influxdb_client.Point(DEST_MEASUREMENT)
            
            if "host" in row.values:            
                p.tag("host", row.values['host'])
            if "domain" in row.values:
                p.tag("domain", row.values['domain'])
            if "page" in row.values:
                p.tag("page", row.values['page'])
            if "action" in row.values:
                p.tag("action", row.values['action'])    
                
            p.field(key, row.values['_value'])
            p.time(row.values['_time'])
            write_api.write(DEST_BUCKET, DEST_ORG, p)

# Setup
client = influxdb_client.InfluxDBClient(url="http://telegrafexternal.bentasker.co.uk", token="", org="")
write_api = client.write_api(write_options=SYNCHRONOUS)
query_api = client.query_api()

START="-3h"

# Pull platform info
q = '''
from(bucket: "telegraf/autogen")
  |> range(start: ''' + START + ''')
  |> filter(fn: (r) => r._measurement == "pf_analytics_test")
  |> filter(fn: (r) => r._field == "platform")
  |> map(fn: (r) => ({ r with _field: r._value}))
  |> drop(columns: ["sess_id"])
  |> aggregateWindow(every: 15m, fn: count)
  |> map(fn: (r) => ({r with platform: r._field, _field: "viewcount"})) 
'''
query_and_write_agg_info(q,"platform", query_api, write_api, 'platform_')


        
# Pull Referrering domain info        
q = '''  
import "strings"

from(bucket: "telegraf/autogen")
  |> range(start: ''' + START + ''')
  |> filter(fn: (r) => r._measurement == "pf_analytics_test")
  |> filter(fn: (r) => r._field == "referrer_domain")
  |> filter(fn: (r) => r._value != "")
  |> map(fn: (r) => ({ r with _field: 
                       if (strings.hasPrefix(v: r._value, prefix: "www.google")) then "google" 
                       else if (strings.hasSuffix(v: r._value, suffix: "reddit.com")) then "reddit" 
                       else if (strings.hasSuffix(v: r._value, suffix: "github.com")) then "github"
                       else if (strings.hasSuffix(v: r._value, suffix: "qwant.com")) then "qwant" 
                       else if (strings.hasSuffix(v: r._value, suffix: "facebook.com")) then "facebook"
                       else if (strings.hasSuffix(v: r._value, suffix: "reddit.com")) then "reddit" 
                       else if (strings.hasSuffix(v: r._value, suffix: "search.yahoo.com")) then "yahoo" 
                       else if (strings.hasSuffix(v: r._value, suffix: "yandex.ru")) then "yandex" 
                       else if (strings.hasSuffix(v: r._value, suffix: "bing.com")) then "bing" 
                       else if (strings.hasSuffix(v: r._value, suffix: "startpage.com")) then "startpage" 
                       else if (r._value == "t.co") then "twitter"
                       else if (strings.hasSuffix(v: r._value, suffix: "twitter.com")) then "twitter" 
                       else if (strings.hasSuffix(v: r._value, suffix: "youtube.com")) then "youtube" 
                       else if (r._value == "juhanurmihxlp77nkq76byazcldy2hlmovfu2epvl5ankdibsot4csyd.onion") then "ahmia"
                       else if (strings.hasSuffix(v: r._value, suffix: "ahmia.fi")) then "ahmia"
                       else if (strings.hasSuffix(v: r._value, suffix: "onion.city")) then "tor2web"
                       else if (strings.hasSuffix(v: r._value, suffix: "onion.direct")) then "tor2web"
                       else if (strings.hasSuffix(v: r._value, suffix: "onion.ly")) then "tor2web"
                       else if (strings.hasSuffix(v: r._value, suffix: "onion.ws")) then "tor2web"
                       else if (strings.hasSuffix(v: r._value, suffix: "onion.dog")) then "tor2web"
                       else if (strings.hasSuffix(v: r._value, suffix: "onion.fi")) then "tor2web"
                       else if (strings.hasSuffix(v: r._value, suffix: "onion.cab")) then "tor2web"
                       else if (strings.hasSuffix(v: r._value, suffix: "onion.pet")) then "tor2web"
                       else if (strings.hasSuffix(v: r._value, suffix: "onion.to")) then "tor2web"
                       
                       
                       else r._value}))
  |> drop(columns: ["sess_id"])
  |> aggregateWindow(every: 15m, fn: count)
  |> map(fn: (r) => ({r with referringdomain: r._field, _field: "viewcount"}))  
'''
query_and_write_agg_info(q,"referringdomain", query_api, write_api, 'referrer_')


q = '''
from(bucket: "telegraf/autogen")
  |> range(start: ''' + START + ''')
  |> filter(fn: (r) => r._measurement == "pf_analytics_test")
  |> filter(fn: (r) => r._field == "timezone")


  |> map(fn: (r) => ({ r with _field: string(v: r._value)}))
  |> drop(columns: ["sess_id"])
  |> aggregateWindow(every: 15m, fn: count)
  |> map(fn: (r) => ({r with timezone: r._field, _field: "tz_pageviews"})) 
'''
query_and_write_agg_info(q,"timezone", query_api, write_api, 'tz_')


q = '''
from(bucket: "telegraf/autogen")
  |> range(start: ''' + START + ''')
  |> filter(fn: (r) => r._measurement == "pf_analytics_test" and r._field == "response_time")
  |> drop(columns: ["sess_id"])
  |> aggregateWindow(every: 15m, fn: mean)
  |> group()

'''
query_and_write_field_val(q, "response_time", query_api, write_api)

q = '''
from(bucket: "telegraf/autogen")
  |> range(start: ''' + START + ''')
  |> filter(fn: (r) => r._measurement == "pf_analytics_test" and r._field == "response_time")
  |> drop(columns: ["sess_id"])
  |> aggregateWindow(every: 15m, fn: max)
  |> group()

'''
query_and_write_field_val(q, "max_response_time", query_api, write_api)

q = '''
from(bucket: "telegraf/autogen")
  |> range(start: ''' + START + ''')
  |> filter(fn: (r) => r._measurement == "pf_analytics_test" and r._field == "response_time")
  |> drop(columns: ["sess_id"])
  |> aggregateWindow(every: 15m, fn: max)
  |> group()

'''
query_and_write_field_val(q, "min_response_time", query_api, write_api)


q = '''
from(bucket: "telegraf/autogen")
  |> range(start: ''' + START + ''')
  |> filter(fn: (r) => r._measurement == "pf_analytics_test" and r._field == "response_time")
  |> drop(columns: ["sess_id"])
  |> aggregateWindow(every: 15m, fn: count)
  |> group()

'''
query_and_write_field_val(q, "requests", query_api, write_api)


# Search related stats

q = '''
from(bucket: "telegraf/autogen")
  |> range(start: ''' + START + ''')
  |> filter(fn: (r) => r._measurement == "pf_analytics_test_search_terms")
  |> filter(fn: (r) => r._field == "search_term")
  |> drop(columns: ["sess_id"])
  |> aggregateWindow(every: 15m, fn: count)
  |> map(fn: (r) => ({r with _field: "search_count", action: "search"}))
  
'''

query_and_write_field_val(q, "search_count", query_api, write_api)

q = '''
from(bucket: "telegraf/autogen")
  |> range(start: ''' + START + ''')
  |> filter(fn: (r) => r._measurement == "pf_analytics_test_search_terms")
  |> filter(fn: (r) => r._field == "result_count")
  |> drop(columns: ["sess_id"])
  |> aggregateWindow(every: 15m, fn: mean)
  |> group()
  
'''
query_and_write_field_val(q, "average_search_result_count", query_api, write_api)


# 404s (websites/privacy-sensitive-analytics#11)
q = '''
from(bucket: "telegraf/autogen")
  |> range(start: ''' + START + ''')
  |> filter(fn: (r) => r._measurement == "pf_analytics_test_404s")
  |> filter(fn: (r) => r._field == "response_time")
  |> keep(columns: ["_time","domain","_value","host"])
  |> aggregateWindow(every: 15m, fn: count)
  |> group()
'''
query_and_write_field_val(q, "404_count", query_api, write_api)



# Bad domains (websites/privacy-sensitive-analytics#12)
# We write these into a seperate measurement, so temporarily override
DEST_MEASUREMENT_OLD = DEST_MEASUREMENT

DEST_MEASUREMENT = "pf_analytics_bad_domains"

q = '''
from(bucket: "telegraf/autogen")
  |> range(start: ''' + START + ''')
  |> filter(fn: (r) => r._measurement == "pf_analytics_test_unauth" and r._field == "domain")
  |> map(fn: (r) => ({ _time: r._time, ref: r._value, _value: 1, host: r.host }))
  |> group()
  |> aggregateWindow(every: 15m, fn: sum)
'''
query_and_write_field_val(q, "bad_domain_requests", query_api, write_api)

q = '''
from(bucket: "telegraf/autogen")
  |> range(start: ''' + START + ''')
  |> filter(fn: (r) => r._measurement == "pf_analytics_test_unauth" and r._field == "domain")
  |> map(fn: (r) => ({ _time: r._time, _value: r._value, host: r.host}))
  |> window(every: 15m)
  |> distinct()
  |> count()
  |> map(fn: (r) => ({ _time: r._stop, host: r.host, _value: r._value}))
  |> group()
'''
query_and_write_field_val(q, "unique_bad_domains", query_api, write_api)


q = '''
import "strings"
from(bucket: "telegraf/autogen")
  |> range(start: ''' + START + ''')
  |> filter(fn: (r) => r._measurement == "pf_analytics_test_enhanced_unauth" and r._field =="domain")
  |> map(fn: (r) => ({
                _time: r._time,
                _field: r._field,
                _value: strings.joinStr(arr: [r._value,
                                              strings.trim(v: r.page, cutset: "\\"")
                                              ], v: ""),
                host: r.host,
              }))
'''
query_and_write_field_val(q, "bad_domain_pages", query_api, write_api)


q = '''
import "strings"
from(bucket: "telegraf/autogen")
  |> range(start: ''' + START + ''')
  |> filter(fn: (r) => r._measurement == "pf_analytics_test_enhanced_unauth" and r._field =="domain")
  |> map(fn: (r) => ({
                _time: r._time,
                _field: r._field,
                _value: r._value,
                host: r.host,
              }))
  
'''
query_and_write_field_val(q, "bad_domain_domains", query_api, write_api)


DEST_MEASUREMENT = DEST_MEASUREMENT_OLD
# Bad domains END


# Keep a log of search terms - the aim is to be able to do semantic processing at some point
q = '''
from(bucket: "telegraf/autogen")
  |> range(start: ''' + START + ''')
  |> filter(fn: (r) => r._measurement == "pf_analytics_test_search_terms")
  |> filter(fn: (r) => r._field == "search_term")
  |> group()
'''

# We write these into a seperate bucket, so temporarily override
DEST_MEASUREMENT_OLD = DEST_MEASUREMENT
DEST_BUCKET_OLD = DEST_BUCKET

DEST_MEASUREMENT = "search_terms"
DEST_BUCKET = "search_terms/autogen"

query_and_write_field_val(q, "search_term", query_api, write_api)

DEST_MEASUREMENT = DEST_MEASUREMENT_OLD
DEST_BUCKET = DEST_BUCKET_OLD


